Attribute VB_Name = "LEF_Open_2017_08_02"
Option Explicit
Option Compare Text ' Non-case sensitive

'   * Copyright  2009-2014 by John P. Desautels (John@SmartBizNiz.com)
'   * Permission is granted to any individual/institution/corporate entity to use, copy, redistribute
'   * or modify this software for any purpose whatsoever, subject to one condition: namely, that this
'   * copyright block, in its entirety, not be removed from the source code.
'   *
'   * Source code, samples and documentation can be found at http://www.SmartBizNiz.com/downloads.html
'   *
'   * Special thanks to:
'   *   Ted Rau who introduced me to Launch Excel and inspired me to develop and share this framework
'   *   Guy Brown who helped me as I developed intricate YSL reports, which are the genesis of this framework
'   *   Deborah Brown, my first Yardi report writing instructor, who explained the history behind Launch Excel
'   *   Geoffrey Gelb who provided interesting projects which have allowed me to enhance the framework in support
'   *       of their requirements
'   *   All the Yardi VBA developers who have come before me, paving the way for this framework

' Set to True during development to Stop the code
'#Const DEBUG_ = True
#Const DEBUG_ = False

Public myCSV As String                       ' Name of CSV file with data
Public myCSVPath As String
Public myXLS As String
Public strLastGoodStatement As String        ' Last successful procedure call. Used for error tracking.
Public boolCloseMe As Boolean
Public myMode As String
Public myOutput As String
Public myOutputFile As String
Public myOrientation As String
Public myPrinter As String


Sub Auto_Open()

' 2009-02-11 JPD    Created based on sample from Ted Rau
' 2009-07-31 JPD    Enhanced myCSV logic to confirm that the CSV is in a path ending in YSI.NetClient
' 2009-11-17 JPD    Enhanced logic to allow for processing of reports via Conductor
' 2009-12-08 JPD    Changed datatypes from Integer to Long to support larger data sets
' 2010-03-06 JPD    Added Replace(Replace(Workbooks(myXLS).FullName, "http:", ""), "/", "\") = myOutputFile
' 2010-03-07 JPD    Completed support for printing via Conductor
' 2011-04-14 JPD    Added code to remove Chr(160) (nonbreaking spaces) that Conductor.Net seems to be adding after certain text while opening CSV file
' 2011-04-14 JPD    Enhance the logic for printing via Conductor
' 2013-01-11 JPD    Added Unicode support
' 2014-09-15 JPD    Added Silverlight support
' 2014-10-23 JPD    Refined Silverlight
' 2014-11-20 JPD    Added replacement of keyword #vbLf# with vbLf - Voyager 7s actually creates additional rows in the CSV file.

On Error GoTo ErrorHandler

    Dim Wkb As Workbook
    Dim Wks As Worksheet
    Dim WkbNew As Workbook
    Dim WksNew As Worksheet
    Dim CurrentXlCalculation As XlCalculation
    Dim CurrentWorksheet As String
    Dim strErrorMessage As String
    Dim iFile As Long
    Dim intCurrentRow As Long
    Dim i As Long
    Dim intSelected As Long
    Dim strReport As String
    Dim boolUTF8 As Boolean
    Dim myCSVFullName As String
    Dim myUnicodeCSVFullName As String
    Dim myUnicodeCSV As String
    
    ' If we're in debug mode then this stops the code so we can step through it
    #If DEBUG_ Then
        Stop
    #End If

    ' Store the current worksheet that is being displayed.
    CurrentWorksheet = ThisWorkbook.ActiveSheet.Name

    ' Display the Message sheet, if it exists
    If WorksheetExists(ThisWorkbook.Name, "Message") Then
        ThisWorkbook.Worksheets("Message").Visible = True
        ThisWorkbook.Worksheets("Message").Activate
    End If
    strLastGoodStatement = "ThisWorkbook.Worksheets(""Message"").Activate"

    With Application
        ' Store the current calculation method. We will restore later.
        CurrentXlCalculation = .Calculation
        ' Temporarily disable certain Excel functionality
        .Calculation = xlManual
        .ScreenUpdating = False
        .DisplayAlerts = False
    End With

   ' Set myXLS as the current workbook
   myXLS = ThisWorkbook.Name

   strLastGoodStatement = "myXLS = ThisWorkbook.Name"

    ' Find the first open CSV file that is in the correct path. This will be our data source.
      For Each Wkb In Application.Workbooks
        ' Launch Excel CSV files are in the YSI.NetClient folder
        If UCase(Right(Wkb.Name, 3)) = "CSV" And UCase(Right(Wkb.Path, 13)) = UCase("YSI.NetClient") Then
          myCSV = Wkb.Name
          myCSVFullName = Wkb.FullName
          myCSVPath = Wkb.Path
          myMode = "Launch Excel"
          Exit For
        ' Silverlight stores in Temp folder
        ElseIf UCase(Right(Wkb.Name, 3)) = "CSV" And Wkb.Path = GetLongFilename(Environ("TEMP")) Then
          myCSV = Wkb.Name
          myCSVFullName = Wkb.FullName
          myCSVPath = Wkb.Path
          myMode = "Launch Excel Silverlight"
          Exit For
        ' Non-Launch Excel CSV files have the same name as myXLS
        ElseIf UCase(Right(Wkb.Name, 3)) = "CSV" And UCase(Left(Wkb.Name, Len(Wkb.Name) - 4)) = UCase(Left(myXLS, Len(myXLS) - 4)) Then
          myCSV = Wkb.Name
          myCSVFullName = Wkb.FullName
          myCSVPath = Wkb.Path
          ' Could be either Conductor or Enterprise. We'll confirm later.
          myMode = "Other"
          Exit For
        End If
      Next

    strLastGoodStatement = "myCSV = Wkb.Name"

    ' If there is no CSV file, we're done
    If myCSV = "" Then
        GoTo ExitProcedure
    End If
    
    ' 2013-01-11 JPD Added Unicode support
    ' Keyword <UTF-8=Yes>. If missing, default to false
    If FirstMatch("<UTF-8=Yes>", myCSV, Workbooks(myCSV).Worksheets(1).Name, "A:A") = -1 Then
        boolUTF8 = False
    Else
        boolUTF8 = True
    End If
    
    If boolUTF8 Then
        ' Create a name for the new CSV file
        myUnicodeCSV = Left(myCSV, Len(myCSV) - 4) & "UTF-8.csv"
        myUnicodeCSVFullName = Left(myCSVFullName, Len(myCSVFullName) - 4) & "UTF-8.csv"
        ' Close myCSV
        Workbooks(myCSV).Close
        ' Create the Unicode CSV file
         If CreateUnicodeCSV("UTF-8", myCSVFullName, myUnicodeCSVFullName, True) Then
            ' The new file will now be our CSV file
            myCSV = myUnicodeCSV
            myCSVFullName = myUnicodeCSVFullName
            strLastGoodStatement = "CreateUnicodeCSV"
         End If
        ' Open myCSV
        Workbooks.Open myCSVFullName
        strLastGoodStatement = "Open Unicode CSV = " & myCSVFullName
        ' Make myXLS be active again
         Workbooks(myXLS).Activate
    End If

    ' Placeholder for custom code to be run before Call CopyCSV
    Call Custom_Open_CopyCSV_Before(myXLS)

    ' Copy the data from myCSV to the CSV sheet in myXLS
    Call CopyCSV

    strLastGoodStatement = "Call CopyCSV"
    
    ' 2011-04-14 JPD Remove Chr(160) (nonbreaking spaces) that Conductor.Net seems to be adding after certain text while opening CSV file
    Application.Workbooks(myXLS).Sheets("CSV").UsedRange.Replace What:=Chr(160), LookAt:=xlPart, Replacement:=""
    
    strLastGoodStatement = "Remove Chr(160) (nonbreaking spaces) from CSV tab"

    ' 2011-11-20 JPD Replace Keyword #vbLf# with vbLf
    Application.Workbooks(myXLS).Sheets("CSV").UsedRange.Replace What:="#vbLf#", LookAt:=xlPart, Replacement:=vbLf
    
    strLastGoodStatement = "Replace #vbLf# with vbLf in CSV tab"

    ' Placeholder for custom code to be run after Call CopyCSV
    Call Custom_Open_CopyCSV_After(myXLS)

   ' Close the CSV file
    Windows(myCSV).Close

    strLastGoodStatement = "Windows(myCSV).Close"

    Windows(myXLS).Activate

    ' Delete the Errors worksheet, if it exists
    If WorksheetExists(myXLS, "Errors") Then
        Workbooks(myXLS).Worksheets("Errors").Delete
        strLastGoodStatement = "Workbooks(myXLS).Worksheets(""Errors"").Delete"
    End If

    ' Test for data present in reports processed by Conductor
    If myMode = "Other" Then
        With Workbooks(myXLS).Worksheets("CSV")
            If UCase(.Cells(1, 1)) = "PRINT" Then
                myMode = "Conductor"
                myOutput = "Printer"
                ' We probably don't care about orientation, but will capture it anyway
                If UCase(.Cells(1, 2)) = "LANDSCAPE" Then
                    myOrientation = "Landscape"
                Else
                    myOrientation = "Portrait"
                End If
                myPrinter = Trim(.Cells(1, 3))
            ElseIf InStr(.Cells(1, 1), ":\") > 0 Or Left(.Cells(1, 1), 1) = "\" Then
                myMode = "Conductor"
                myOutput = "File"
                myOutputFile = .Cells(1, 1)
            Else
                myMode = "Enterprise"
            End If
        End With
    End If

    ' Placeholder for custom code to be run before Call BuildReport
    Call Custom_Open_BuildReport_Before(myXLS)

    ' Now execute any code specific to this report
    Call BuildReport

    ' Placeholder for custom code to be run after Call BuildReport
    Call Custom_Open_BuildReport_After(myXLS)

    ' If this is a Conductor report then print or save
    If myMode = "Conductor" Then
        Select Case myOutput
            Case "Printer"
                ' Select all of the worksheets in the LEFReports tab and then print them out
                With Workbooks(myXLS).Worksheets("LEFReports")
                    intCurrentRow = 2 ' This is the first row of data
                    intSelected = 0 ' Initialize
                    
                    For i = intCurrentRow To .UsedRange.Rows.Count
                        ' Report name is in the first column
                        strReport = .Cells(i, 1).Value
                        
                        ' Test to be sure the report still exists
                        If WorksheetExists(myXLS, strReport) Then
                            If intSelected = 0 Then
                                ' Select the first sheet and Activate
                                Workbooks(myXLS).Sheets(strReport).Select
                                Workbooks(myXLS).Sheets(strReport).Activate
                            Else
                                ' Add this sheet to the selected sheets
                                Workbooks(myXLS).Sheets(strReport).Select (False)
                            End If
                            
                            intSelected = intSelected + 1
                        End If
                    
                    Next
                End With
                ' Print the selected worksheets
                ActiveWindow.SelectedSheets.PrintOut ActivePrinter:=myPrinter, Copies:=1, Collate:=True
                ' Quit
                Application.Quit
            Case "File"
                ' Test to see if we have already done a SaveAs in BuildReport
                ' 2010-03-06 JPD When processed on a local webserver the path may converted from UNC to URL.
                '                  These replacements should correct that.
                If Replace(Replace(Workbooks(myXLS).FullName, "http:", ""), "/", "\") = myOutputFile Then
                    ' Nothing to do
                ' Test to see if the file already exists. This should never happen with Conductor.
                ElseIf Dir(myOutputFile) <> "" Then
                    strErrorMessage = "The output file " & myOutputFile & " exists and cannot be overwritten."
                Else
                    ' Create a new workbook. For some reason Conductor doesn't like SaveAs
                    Set WkbNew = Workbooks.Add
                    strLastGoodStatement = "WkbNew = Workbooks.Add"
                    'Copy the CSV. If the report wasn't saved during BuildReport then this is all we will do
                    Application.Workbooks(myXLS).Sheets("CSV").Copy before:=WkbNew.Worksheets("Sheet1")
                    ' Remove empty worksheets
                    For Each WksNew In WkbNew.Worksheets
                        If Left(WksNew.Name, 5) = "Sheet" Then
                            WksNew.Delete
                            strLastGoodStatement = "WksNew.Delete"
                        End If
                    Next WksNew

                    ' Copy any errors to the new workbook
                    If WorksheetExists(myXLS, "Errors") Then
                        Application.Workbooks(myXLS).Sheets("Errors").Copy before:=WkbNew.Worksheets(1)
                        strLastGoodStatement = "Application.Workbooks(" + myXLS + ").Sheets(""Errors"").Copy before:=WkbNew.Worksheets(1)"
                    End If

                    ' Activate the first worksheet in the new book. It will be the errors or the first report
                    WkbNew.Worksheets(1).Activate
                    WkbNew.Activate
                    ' Save the new workbook
                    WkbNew.SaveAs FileName:=myOutputFile
                End If
        End Select
    End If

ExitProcedure:
    Set WkbNew = Nothing
    
    If myMode = "Conductor" Then
        ' Restore the original calculation method.
        Application.Calculation = CurrentXlCalculation
        ' Close without prompting to save changes
        ActiveWorkbook.Close savechanges:=False
        Application.Quit
    Else
        ' Placeholder for custom code to be run at the start of ExitProcedure
        Call Custom_Open_ExitProcedure_Before(myXLS)
    
        With Application
            ' Restore the original calculation method.
            .Calculation = CurrentXlCalculation
            ' Restore temporarily disabled functionality
            .DisplayAlerts = True
            .ScreenUpdating = True
            ' TODO: See why Ted included .CalculateBeforeSave = False
            '.CalculateBeforeSave = False
        End With
    
        ' It's time to go
        If boolCloseMe Then
            ' Flag as saved to avoid the prompt asking to save changes
            ThisWorkbook.Saved = True
            ' 2013-06-21 JPD Added savechanges:=False. In some cases the user was still be prompted to save the file.
            ThisWorkbook.Close savechanges:=False
    
        Else
            If WorksheetExists(ThisWorkbook.Name, "Errors") Then
                ' Activate the Errors sheet, if it exists
                ThisWorkbook.Worksheets("Errors").Activate
            ElseIf WorksheetExists(ThisWorkbook.Name, CurrentWorksheet) Then
                ' Activate the original sheet, if it still exists
                ThisWorkbook.Worksheets(CurrentWorksheet).Activate
            Else
                ' Otherwise, get the first one
                ThisWorkbook.Worksheets(1).Activate
            End If
        End If
    
        ' Placeholder for custom code to be run at the end of ExitProcedure
        Call Custom_Open_ExitProcedure_After(myXLS)

    End If
    
    Exit Sub

ErrorHandler:
    Select Case Err.Number
        Case Else
            strErrorMessage = Err.Number & ": " & Err.Description & vbCrLf & _
                "Last good statement: " & strLastGoodStatement

            Call LogError("Auto_Open", strErrorMessage)

            #If DEBUG_ Then
                'Message boxes only in DEBUG mode
                MsgBox strErrorMessage, vbOKOnly, "Auto_Open"
            #End If

    End Select

    If myMode = "Conductor" Then
        'write error to file for debugging
        If strErrorMessage = "" Then strErrorMessage = Err.Description
        If Trim(strErrorMessage) = "" Then Exit Sub
        On Error Resume Next
        iFile = FreeFile
        Open "c:\" & Mid(myXLS, 1, Len(myXLS) - 4) & ".txt" For Output As #iFile
        Print #iFile, "Error Msg: " & Left(strErrorMessage, 500) & "; Outfile: " & myOutputFile & "; Last Good Stmt: " & strLastGoodStatement
        Close #iFile
    End If

    Resume ExitProcedure

End Sub
Private Sub CopyCSV()
    ' 2009-02-11 JPD    Created

    ' .UsedRange was added because:
    '  - Excel 2007 was opening myCSV and Excel 2003 was opening myXLS.
    '    Attempting to copy all cells from Excel 2007 into 2003 resulted in an error
    '    because there were too many rows.
    '  - This may never happen to anyone but me.


    With Workbooks(myXLS)
        If Not WorksheetExists(myXLS, "CSV") Then
            ' Add CSV worksheet to myXLS if it doesn't exist
            .Worksheets.Add(after:=Worksheets(.Worksheets.Count)).Name = "CSV"
        Else
            ' Otherwise, clear any existing data
            .Worksheets("CSV").UsedRange.Clear
        End If

        ' Copy the data from the CSV file to cell A1
        Workbooks(myCSV).Worksheets(1).UsedRange.Copy .Worksheets("CSV").Cells(1, 1)

    End With

End Sub

